﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Maze
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }


        Button[,] walls = new Button[20, 20];
        const int WALL_SIZE = 20;

        private void Form1_Load(object sender, EventArgs e)
        {
            System.IO.StreamReader reader = new System.IO.StreamReader("maze.txt",Encoding.ASCII);
            String savedWalls = reader.ReadToEnd();

            int savedWallsIndex = 0;
            for (int row = 0; row < 20; row++)
            {
                for (int column = 0; column < 20; column++)
                {
                    Button newButton = new Button();
                    newButton.BackColor = Color.White;
                    newButton.Top = row * WALL_SIZE + 30;
                    newButton.Left = column * WALL_SIZE;
                    newButton.Width = WALL_SIZE;
                    newButton.Height = WALL_SIZE;
                    walls[row, column] = newButton;
                    Controls.Add(newButton);
                    newButton.Click += new System.EventHandler(this.wallClicked);

                    if (savedWalls[savedWallsIndex] == 'W')
                    {
                        newButton.BackColor = Color.Black;
                    }
                    else if (savedWalls[savedWallsIndex] == 'D')
                    {
                        newButton.BackColor = Color.Red;
                    }
                    else if (savedWalls[savedWallsIndex] == 'S')
                    {
                        newButton.BackColor = Color.Green;
                    }

                    savedWallsIndex++;
                }
            }
        }

        private void wallClicked(object sender, EventArgs e)
        {
            Button button = (Button)sender;
            if (button.BackColor == Color.White)
            {
                button.BackColor = Color.Black;
            }
            else if (button.BackColor == Color.Black)
            {
                button.BackColor = Color.Green;
            }
            else if (button.BackColor == Color.Green)
            {
                button.BackColor = Color.Red;
            }
            else
            {
                button.BackColor = Color.White;
            }
            
        }

        private void menuStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {

        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            System.IO.FileStream file = new System.IO.FileStream("maze.txt",System.IO.FileMode.Create);

            for (int row = 0; row < 20; row++)
            {
                for (int column = 0; column < 20; column++)
                {
                    if (walls[row, column].BackColor == Color.Black)
                    {
                        file.WriteByte((byte)'W');
                    }
                    else if (walls[row, column].BackColor == Color.Red)
                    {
                        file.WriteByte((byte)'D');
                    }
                    else if (walls[row, column].BackColor == Color.Green)
                    {
                        file.WriteByte((byte)'S');
                    }
                    else
                    {
                        file.WriteByte((byte)' ');
                    }
                }
            }
            file.Close();
        }

        private Position getPosition(Color color)
        {
            Position result = new Position();

            for (int row = 0; row < 20; row++)
            {
                for (int column = 0; column < 20; column++)
                {
                    if(walls[row,column].BackColor == color)
                    {
                        result.row = row;
                        result.column = column;
                        result.direction = 0;
                    }
                }
            }
            return result;
        }

        private void dropBreadCrumb(Position current)
        {
            if (walls[current.row, current.column].BackColor == Color.White ||
                walls[current.row, current.column].BackColor == Color.Brown)
            {
                walls[current.row, current.column].BackColor = Color.Wheat;
            }
        }

        private void eatBreadCrumb(Position current)
        {
            if (walls[current.row, current.column].BackColor == Color.Wheat)
            {
                walls[current.row, current.column].BackColor = Color.Brown;
            }
        }

        private void btnSolve_Click(object sender, EventArgs e)
        {
            Position start = getPosition(Color.Green);
            Position current = getPosition(Color.Green);
            Position destination = getPosition(Color.Red);

            System.Collections.Generic.Stack<Position> history = 
                new System.Collections.Generic.Stack<Position>();

            do
            {
                bool moveMade = false;
                do
                {
                    //add comments
                    if (current.direction == 0 && 
                        (walls[current.row - 1, current.column].BackColor == Color.White ||
                        walls[current.row - 1, current.column].BackColor == Color.Red))
                    {
                        history.Push(new Position(current));
                        dropBreadCrumb(current);
                        current.row--;
                        current.direction = 0;
                        moveMade = true;
                    }
                    else if (current.direction == 1 &&
                       ( walls[current.row, current.column + 1].BackColor == Color.White ||
                        walls[current.row, current.column + 1].BackColor == Color.Red))
                    {
                        history.Push(new Position(current));
                        dropBreadCrumb(current);
                        current.column++;
                        current.direction = 0;
                        moveMade = true;
                    }
                    else if (current.direction == 2 &&
                        (walls[current.row + 1, current.column].BackColor == Color.White ||
                        walls[current.row + 1, current.column].BackColor == Color.Red))
                    {
                        history.Push(new Position(current));
                        dropBreadCrumb(current);
                        current.row++;
                        current.direction = 0;
                        moveMade = true;
                    }
                    else if (current.direction == 3 &&
                        (walls[current.row, current.column - 1].BackColor == Color.Red ||
                        walls[current.row, current.column - 1].BackColor == Color.White))
                    {
                        history.Push(new Position(current));
                        dropBreadCrumb(current);
                        current.column--;
                        current.direction = 0;
                        moveMade = true;
                    }
                    if(!moveMade )
                    {
                        current.direction++;
                    }

                }
                while(current.direction < 4 && !moveMade);

                if (!moveMade)
                {
                    walls[current.row, current.column].BackColor = Color.Brown;
                    current = history.Pop();
                    eatBreadCrumb(current);
                    current.direction++;
                }
                System.Threading.Thread.Sleep(100);
                foreach (Control control in Controls)
                {
                    control.Refresh();
                }
            }
            while (walls[current.row, current.column].BackColor != Color.Red &&
                (history.Count != 0 || current.direction < 4)  );
 //while (!(current.row == destination.row && 
 //               current.column == destination.column) );

        }
    }

    class Position
    {
        public int row;
        public int column;
        public int direction;

        public Position()
        {
        }

        public Position(Position positionToCopy)
        {
            row = positionToCopy.row;
            column = positionToCopy.column;
            direction = positionToCopy.direction;
        }
    }
}
